// --- [ hyt224.c ] -----------------------------------------------------------
//

#include <avr/io.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "TWI_Master_c.h"
#include "hyt_224.h"
#include "global.h"
#include "rs232.h"

void Make_Dez_Point(void);

// ----------------------------------------------------------------------------
// liest die Werte aus dem Modul aus - 
// falls die Daten noch nicht bereit sind, dann starten weitere Leseversuche.
// Die eingelesenen Wert werden umgerechnet um mit 1 Nachkommastelle ausgegeben

void HYT224_read(void)
{
uint32_t t;
int16_t  temperatur;
uint16_t humidity;
uint8_t  buffer[6];
uint8_t  i = 10;

while(1)
	{
	buffer[0] = HYT224_ADR;						// Adresse fr TWI-Write
	TWI_MA_Start_Transceiver_With_Data(buffer, 1);

// dieser Test kann auch entfallen
	if (!(TWI_LastTransOK()))					// Prfen, ob Slave ein ACK gesendet hat
		{
		puts_p(PSTR("TWI-Write-Error"));
		put_c(13);
		return;
		}
		
	delay_10ms(10);								// Wartezeit 100ms (50ms reicht nicht aus)

	buffer[0]++;									// Adresse fr TWI-Read incrementieren
	TWI_MA_Start_Transceiver_With_Data(buffer, 5); // Daten via TWI einlesen
	TWI_MA_Get_Data_From_Transceiver(buffer, 5);// Daten abholen
	
// dieser Test kann auch entfallen
	if (!(TWI_LastTransOK()))
		{
		puts_p(PSTR("TWI-Read-Error"));
		put_c(13);
		return;
		}
										
	if (buffer[1]  & 0b11000000) 				// Bit.7/6 des ersten Datenbytes testen
		{
		i--;											// wenn gesetzt, dann ist der Sensor noch busy
		put_c('+');									// ein '+' plus Flags hexadezimal ausgeben
		put_s(utoa((buffer[1] & 0b11000000), cv, 16));
		put_c(' ');									// wenn diese Ausgabe erfolgt, dann war die Wartezeit zu kurz !
		}

	else												// wenn Bit.7/6 nicht gesetzt sind
		{
		break;										// die Schleife while(1) beenden
		}
		
	if (!(i))										// wenn i-Versuche erfolglos waren,
		{												// dann beenden
		puts_p(PSTR("Modul busy"));
		put_c(13);
		return;										// Abbrechen
		}
	}
// -------------------------------------  zur Kontrolle die Messwerte ausgeben
for (i = 1; i < 5; i++)							// ohne die TWI-Adresse in buffer[0] !
	{
	put_s(utoa(buffer[i], cv, 10));
	put_c(' ');
	}

put_c(':');
put_c(' ');
// ------------------------------------- 	Humidity konvertieren
t  = (buffer[1] << 8);								
t +=  buffer[2];
t *= 1000UL;										// mit 100 * 10 multiplizieren fr
t /= 16384UL;										// eine zustzliche Nachkommastelle !

humidity = (uint16_t) t; 

utoa(humidity, cv, 10);							// die Zahl in String in cv[] umwandeln

// und ein Dezimaltrennzeichen einfgen
i = 0;
while(cv[i]) i++;									// zum Stringende durchhangeln
cv[i + 1] = 0;
cv[i] = cv[i-1];
cv[i-1] = SEPARATOR;

put_s(cv);											// Wert ausgeben
put_c(' ');

// ------------------------------------- Temperatur konvertieren
t  = buffer[3] << 8;							
t += buffer[4];
t *= 165;
t /= 16384L;
temperatur = (int16_t) (t - 40);

itoa(temperatur, cv, 10);						// Die Zahl in String in cv[] umwandeln

// und ein Dezimaltrennzeichen einfgen
i = 0;
while(cv[i]) i++;
cv[i + 1] = 0;
cv[i] = cv[i-1];
cv[i-1] = SEPARATOR;

put_s(cv);										// Wert ausgeben
put_c(13);

/*
// hier die Daten binr speichern
put_c(' ');

buffer[0] = (uint8_t) (humidity >> 8);
buffer[1] = (uint8_t) (humidity);
buffer[2] = (uint8_t) (temperatur >> 8);
buffer[3] = (uint8_t) (temperatur);

for (i = 0; i < 4; i++)							// zur Kontrolle ausgeben
	{
	put_s(utoa(buffer[i], cv, 10));
	put_c(' ');
	}

put_c(13);
DF_write_array(buffer, 4);
*/

}

/*
// ----------------------------------------------------------------------------
// fgt zwischen dem letzten und dem vorletzten Zeichen im String cv[]
// einen Dezimalpunkt ein
// kostet bei 2-maligem Aufruf aber mehr Speicher als inline
void Make_Dez_Point(void)
{
uint8_t i = 0;

while(cv[i]) i++;									// zum Stringende durchhangeln
cv[i + 1] = 0;										// den neuen Terminator setzen
cv[i] = cv[i-1];									// das letzte Zeichen nach rechts kopieren
cv[i-1] = SEPARATOR;								// das Dezimaltrennzeichen einfgen
}
*/

// --- [ end of file ] --------------------------------------------------------